# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# connectes.py
#
# Librairie de gestion des connectés
#
###########################################################################

import sys, commands, gzip, os
sys.path.append('/usr/share/eole/controlevnc')

# gestion de l'annuaire
import ldap_utils

# Chose pour avoir la liste de véritables connectés
# (net status sessions) renvoie parfois plusieurs login sur la même IP ...
def get_sessions():
    """Récupère la liste des sessions et fait le ménage
    """
    cmd = "net status sessions parseable"
    lignes = commands.getoutput(cmd).splitlines()
    liste = [ tuple(i.split('\\')) for i in lignes ]
    liste = supprime_doublons(liste)
    return liste

def supprime_doublons(liste):
    """Il arrive que 'net status sessions' renvoie plusieurs utilisateurs
    sur la même machine... On regarde dans /var/log/samba/connexions.log
    pour voir qui s'est connecté en dernier et on vire les autres
    """
    res = []
    ips = [ i[4] for i in liste ]
    for pid, username, prigrp, nom_mach, ip in liste:
        if ips.count(ip) == 1:
            res.append((pid, username, prigrp, nom_mach, ip))
        else:
            if get_last_connected(ip, username) == username:
                res.append((pid, username, prigrp, nom_mach, ip))
    return res

def get_last_connected(ip, username):
    """Renvoie le dernier username connecte sur ip
    """
    liste = get_connexions()
    for ligne in liste:
        if ip == ligne.split()[10]:return ligne.split()[6]
    return username
# Fin Chose

def get_connexions():
    """Renvoie une liste contenant l'historique des connexions, le premier élément
    est la connexion la plus récente
    CONNECTION Jeu 24 mai 2007 10:31 julie.evrard eleves xp-k WinXP 192.168.230.154
    """
    liste = file('/var/log/samba/connexions.log').read().splitlines()
    liste.reverse() # les dernières connexions au début
    if os.path.isfile('/var/log/samba/connexions.log.1.gz'):
        liste2 = gzip.GzipFile('/var/log/samba/connexions.log.1.gz').read().splitlines()
        liste2.reverse()
        liste.extend(liste2)
    return liste

def get_connectes(groupe=None, rgroupe=None):
    """renvoie la liste des connectes avec possibilité de croisement sur 2 groupes
    [PID, USERNAME, PRIMGROUP, NOMMACHINE, IP, OS]
    """
    # récupération des connectés
    liste = []
    if groupe:
        if rgroupe: membres = ldap_utils.get_membres_croise(groupe, rgroupe)
        else: membres = ldap_utils.get_membres(groupe)
        # le groupe est vide
        if not membres: return liste
    else: membres = None
    for pid, username, prigrp, nom_mach, ip in get_sessions():
        try:
            # champs : 0 pid, 1 uid, 2 grp, 3 machine, 4 (ip), 5 os (détecté)
            #pid, username, prigrp, nom_mach, ip = ligne.split('\\')
            if membres and username not in membres: continue
            os = detect_os(ip)
            liste.append((pid, username, prigrp, nom_mach, ip, os))
        except:
            continue
    liste.sort()
    return liste

# des informations depuis une IP
def get_user_from_ip(ip):
    """renvoie les informations de session sur "ip"
    """
    liste = get_connexions()
#    CONNECTION Jeu 24 mai 2007 10:31 julie.evrard eleves xp-k WinXP 192.168.230.154
    for donnee in liste:
        d = donnee.split()
        pid, username, prigrp, nom_mach, ip_adr, os = '0', d[6], d[7], d[8], d[10], d[9]
        if ip_adr == ip: return (pid, username, prigrp, nom_mach, ip, os)
#    return None, None, None, None, None, None
    raise Exception("La station %s n'est pas connectee au domaine"%ip)

def get_nom_machine(ip):
    """renvoie le nom de la machine <ip>
    """
    return get_user_from_ip(ip)[3]

def get_groupe(ip):
    """retourne le groupe de l'utilisateur connecté sur la station "ip"
    """
    # récupération des donnees contenant ip
    return get_user_from_ip(ip)[2]

def get_user(ip):
    """retourne le username de l'utilisateur connecte sur "ip"
    """
    return get_user_from_ip(ip)[1]

# des informations depuis une LOGIN
def get_ip_from_user(user):
    """renvoie l(es) element(s) dont USER==user de la liste get_connectes
    """
#    pid, username, prigrp, nom_mach, ip, os
#    ('13040', 'admin', 'professeurs', 'vm-xp1', '192.168.230.157', 'WinXP')
    res = []
    for pid, username, prigrp, nom_mach, ip_adr, os in get_connectes():
        if user == username: res.append((pid, username, prigrp, nom_mach, ip_adr, os))
    return res

def get_ip(user):
    """retourne l'ip de la station ou est connecte "user"
    """
    res = []
    for i in get_ip_from_user(user):
        # n'est utilisé que dans manage_blocage pas compatible avec Win9x
        if i[5] != 'Win95': res.append(i[4])
    return res

def is_connecte(user):
    connectes = [ i[1] for i in get_connectes()]
    if user in connectes: return True
    return False

def detect_os(ip):
    try:
#    if True:
        for i in get_connexions():
            cur_os, cur_ip = i.split()[9:11]
            if cur_ip == ip:
                if cur_os.lower() == 'vista': return 'Vista'
                elif cur_os.lower() == 'winxp': return 'WinXP'
                elif cur_os.lower() == 'win2k': return 'Win2K'
                elif cur_os.lower() == 'winnt': return 'WinNT'
                else: return 'Win95'
        return None
    except:
        return None

