# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# gest_sessions.py
#
# Librairie pour la gestion des clients Windows
#
###########################################################################
 

import sys, os, time
sys.path.append('/usr/share/eole/controlevnc')

from twisted.python import log
import connectes
import ldap_utils
from cliscribe import Cliscribe
import esu

class Logon:
    def __init__(self):
        """Gestion d'ouverture de session sur les postes clients (windows 2K/XP pour l'instant)
        - gestion VNC
        """

    def logon(self, ip, os_type, bloc):
        self.ip, self.os_type, self.bloc = ip, os_type, bloc
        self.user = connectes.get_user(self.ip)
        self.userlname = ldap_utils.get_user_long_name(self.user)
        self.groups = ldap_utils.get_user_groups(self.user)
        self.sid = ldap_utils.get_user_sid(self.user)
        logon_dict=esu.ESU(self)._esu_dict
        if not logon_dict:
            return ('Err', u'Machine non géré par Esu')
        if not logon_dict['esu_dict']['ESU_GU']: 
            return ('Err', u'Utilisateur non géré par Esu')
        d = Cliscribe(self.ip).logon(logon_dict=logon_dict)
        d.addCallback(self.blocage)
        d.addCallback(self.vnc)
        return d

    def blocage(self, e=False):
#        log.msg('Blocage %s'%e)
#        log.msg('%s %s'%esu.get_drives_type(self))
        if 'eleves' in self.groups:
            return self.bloc.bloc_user(self.user, self.ip, logon=True)
        else:
            # on débloque si pas eleve
            return self.bloc.init_net(self.ip)

    def get_vnc_mode(self):
        """récupère le type de controle défini dans l'EAD
        simple, controle, desactive
        """
        if os.path.isfile('/var/www/ead/config/controle'):
            typecontrole = file('/var/www/ead/config/controle').readlines()[0].strip()
        else: typecontrole = 'simple'
        return typecontrole

    def vnc(self, e=False):
        """active le type de controle sur la station (lu par controle-vnc / gestion-postes.exe)
        et arrête le service winvnc pour éviter des observations intempestives
        """
#        log.msg('Vnc %s'%e)
        d = Cliscribe(self.ip).vnc(action='setinputs', value=None, restart=False)
        d.addCallback(Cliscribe(self.ip).vnc, action='stop')
        d.addCallback(Cliscribe(self.ip).vnc, action='stop_listen')
        mode = self.get_vnc_mode()
        return d.addCallback(Cliscribe(self.ip).vnc, action='set_type', value=mode)

