# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# manage_blocage.py
#
# Librairie de gestion du blocage réseau des stations
#
###########################################################################

from twisted.internet import reactor
from twisted.python import log
import sys
sys.path.append('/usr/share/eole/controlevnc')

import connectes
from cliscribe import Cliscribe
import ldap_utils

class GestBlocage:
    """Gestion des blocages réseau
    """
    def __init__(self):
        self.bloc_dict = dict()
        self.bd_list = { 'reseau' : { 0 : 'Aucun', 1 : 'Tout internet'},
                         'partages' : { 0 : 'Normaux', 1 : 'Aucun lecteur réseau', 2 : 'Seulement le partage "devoirs"'}}
        self.mod_dict = {'netmod' : 0 , 'partmod' : 0}

    def get_bloc_list(self):
        """renvoie la liste des types de blocage
        """
        return self.bd_list
        
    def get_net_bloc(self, user):
        """renvoie le type de blocage reseau pour user
        """
        if self.bloc_dict.has_key(user): return self.bloc_dict[user]['netmod']
        return 0

    def get_part_bloc(self, user):
        """renvoie le type de blocage de partage pour user
        """
        if self.bloc_dict.has_key(user): return self.bloc_dict[user]['partmod']
        return 0

    def get_blocage(self, user):
        if self.bloc_dict.has_key(user): return self.bloc_dict[user]
        return self.mod_dict

    def set_blocage(self, user, netmod, partmod, duree):
        """active le blocage netmod pour user pendant duree
        renseigne le dictionnaire { user : type_blocage }
        """
        netmod, partmod, duree = int(netmod), int(partmod), int(duree)*60
        if netmod == 0 and partmod == 0:
            self.purge_l(user)
        else:
            self.bloc_dict[user] = {'netmod' : netmod , 'partmod' : partmod}
            reactor.callLater(duree, self.purge_l, user)
            self.bloc_user(user) #, netmod, partmod)
        
    def purge_l(self, user):
        """supprime user de self.bloc_dict
        """
        try :del(self.bloc_dict[user])
        except: pass
        self.bloc_user(user) #, 0, 0)

    def bloc_user(self, user, ip=None, logon=False): #netmod, partmod, ip=None):
        """active le blocage sur le poste où est connecté user (s'il est connecté bien sûr)
        """
#        log.msg('bloc_user %s %s %s'%(user, ip, logon))
        if not connectes.is_connecte(user): return #rien a faire si non connecte
        if not ip:
            try: ip = connectes.get_ip(user)
            except: return True
        if type(ip) == str: 
            d = self.bloc_partage(ip=ip, logon=logon) #, partmod)
            d.addCallback(self.bloc_net, ip=ip) #, netmod)
            return d
        elif type(ip) == list and ip:
            d = self.bloc_partage(ip=ip[0], logon=logon) #, partmod)
            d.addCallback(self.bloc_net, ip=ip[0]) #, netmod)
            for i in ip[1:]:
                d.addCallback(self.bloc_partage, ip=i, logon=logon) #, partmod)
                d.addCallback(self.bloc_net, ip=i) #, netmod)
            return d

    def bloc_partage(self, ret=None, ip=None, logon=False): #, partmod=None):
#        log.msg('bloc partage %s'%ip)
        user = connectes.get_user(ip)
        sid = ldap_utils.get_user_sid(user)
        partmod = self.get_part_bloc(user)
        return Cliscribe(ip).bloc(partmod=partmod, sid=sid, logon=logon)

    def bloc_net(self, ret=None, ip=None): #, netmod=None):
#        log.msg('bloc net %s ip=%s'%(ret, ip))
        user = connectes.get_user(ip)
        netmod = self.get_net_bloc(user)
        if netmod == 1:
            cmd = ['ipfw add 200 deny all from me to any']
            cmd.append('ipfw add 2 allow tcp from me to any 135,137,139,445 keep-state')
            cmd.append('ipfw add 3 allow udp from me to any 135,137,139,445 keep-state')
            cmd.append('ipfw add 5 deny tcp from me to 192.168.230.88 80,443 keep-state')
            return Cliscribe(ip).execute(cmd=cmd, hide=True)
        elif netmod == 0:
            return self.init_net(ip)

    def init_net(self, ip):
        cmd = 'ipfw %EOLEPATH%\\wipfw\\wipfw.conf'
        return Cliscribe(ip).execute(cmd=cmd, hide=True)

